@@ -50,7 +50,10 @@ uint64_t trace_clock_read64_monotonic(void)
 {
 	struct timespec ts;
 
-	clock_gettime(CLOCK_MONOTONIC, &ts);
+	if (caa_unlikely(clock_gettime(CLOCK_MONOTONIC, &ts))) {
+		ts.tv_sec = 0;
+		ts.tv_nsec = 0;
+	}
 	return ((uint64_t) ts.tv_sec * 1000000000ULL) + ts.tv_nsec;
 }
 
