@@ -3,21 +3,25 @@
  *
  * LTTng UST filter bytecode validator.
  *
- * Copyright (C) 2010-2012 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * Copyright (C) 2010-2016 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; only
- * version 2.1 of the License.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #define _LGPL_SOURCE
@@ -27,6 +31,7 @@
 
 #include <urcu/rculfhash.h>
 #include "lttng-hash-helper.h"
+#include "string-utils.h"
 
 /*
  * Number of merge points for hash table size. Hash table initialized to
@@ -124,7 +129,8 @@ int merge_point_add_check(struct cds_lfht *ht, unsigned long target_pc,
  * (unknown), negative error value on error.
  */
 static
-int bin_op_compare_check(struct vstack *stack, const char *str)
+int bin_op_compare_check(struct vstack *stack, filter_opcode_t opcode,
+		const char *str)
 {
 	if (unlikely(!vstack_ax(stack) || !vstack_bx(stack)))
 		goto error_empty;
@@ -144,6 +150,29 @@ int bin_op_compare_check(struct vstack *stack, const char *str)
 			goto unknown;
 		case REG_STRING:
 			break;
+		case REG_STAR_GLOB_STRING:
+			if (opcode != FILTER_OP_EQ && opcode != FILTER_OP_NE) {
+				goto error_mismatch;
+			}
+			break;
+		case REG_S64:
+		case REG_DOUBLE:
+			goto error_mismatch;
+		}
+		break;
+	case REG_STAR_GLOB_STRING:
+		switch (vstack_bx(stack)->type) {
+		default:
+			goto error_type;
+
+		case REG_UNKNOWN:
+			goto unknown;
+		case REG_STRING:
+			if (opcode != FILTER_OP_EQ && opcode != FILTER_OP_NE) {
+				goto error_mismatch;
+			}
+			break;
+		case REG_STAR_GLOB_STRING:
 		case REG_S64:
 		case REG_DOUBLE:
 			goto error_mismatch;
@@ -158,6 +187,7 @@ int bin_op_compare_check(struct vstack *stack, const char *str)
 		case REG_UNKNOWN:
 			goto unknown;
 		case REG_STRING:
+		case REG_STAR_GLOB_STRING:
 			goto error_mismatch;
 		case REG_S64:
 		case REG_DOUBLE:
@@ -242,6 +272,8 @@ int bytecode_validate_overflow(struct bytecode_runtime *bytecode,
 	case FILTER_OP_LT_STRING:
 	case FILTER_OP_GE_STRING:
 	case FILTER_OP_LE_STRING:
+	case FILTER_OP_EQ_STAR_GLOB_STRING:
+	case FILTER_OP_NE_STAR_GLOB_STRING:
 	case FILTER_OP_EQ_S64:
 	case FILTER_OP_NE_S64:
 	case FILTER_OP_GT_S64:
@@ -329,6 +361,7 @@ int bytecode_validate_overflow(struct bytecode_runtime *bytecode,
 
 	/* load from immediate operand */
 	case FILTER_OP_LOAD_STRING:
+	case FILTER_OP_LOAD_STAR_GLOB_STRING:
 	{
 		struct load_op *insn = (struct load_op *) pc;
 		uint32_t str_len, maxlen;
@@ -413,8 +446,9 @@ int validate_instruction_context(struct bytecode_runtime *bytecode,
 		char *pc)
 {
 	int ret = 0;
+	const filter_opcode_t opcode = *(filter_opcode_t *) pc;
 
-	switch (*(filter_opcode_t *) pc) {
+	switch (opcode) {
 	case FILTER_OP_UNKNOWN:
 	default:
 	{
@@ -442,49 +476,49 @@ int validate_instruction_context(struct bytecode_runtime *bytecode,
 	case FILTER_OP_BIN_XOR:
 	{
 		ERR("unsupported bytecode op %u\n",
-			(unsigned int) *(filter_opcode_t *) pc);
+			(unsigned int) opcode);
 		ret = -EINVAL;
 		goto end;
 	}
 
 	case FILTER_OP_EQ:
 	{
-		ret = bin_op_compare_check(stack, "==");
+		ret = bin_op_compare_check(stack, opcode, "==");
 		if (ret < 0)
 			goto end;
 		break;
 	}
 	case FILTER_OP_NE:
 	{
-		ret = bin_op_compare_check(stack, "!=");
+		ret = bin_op_compare_check(stack, opcode, "!=");
 		if (ret < 0)
 			goto end;
 		break;
 	}
 	case FILTER_OP_GT:
 	{
-		ret = bin_op_compare_check(stack, ">");
+		ret = bin_op_compare_check(stack, opcode, ">");
 		if (ret < 0)
 			goto end;
 		break;
 	}
 	case FILTER_OP_LT:
 	{
-		ret = bin_op_compare_check(stack, "<");
+		ret = bin_op_compare_check(stack, opcode, "<");
 		if (ret < 0)
 			goto end;
 		break;
 	}
 	case FILTER_OP_GE:
 	{
-		ret = bin_op_compare_check(stack, ">=");
+		ret = bin_op_compare_check(stack, opcode, ">=");
 		if (ret < 0)
 			goto end;
 		break;
 	}
 	case FILTER_OP_LE:
 	{
-		ret = bin_op_compare_check(stack, "<=");
+		ret = bin_op_compare_check(stack, opcode, "<=");
 		if (ret < 0)
 			goto end;
 		break;
@@ -511,6 +545,23 @@ int validate_instruction_context(struct bytecode_runtime *bytecode,
 		break;
 	}
 
+	case FILTER_OP_EQ_STAR_GLOB_STRING:
+	case FILTER_OP_NE_STAR_GLOB_STRING:
+	{
+		if (!vstack_ax(stack) || !vstack_bx(stack)) {
+			ERR("Empty stack\n");
+			ret = -EINVAL;
+			goto end;
+		}
+		if (vstack_ax(stack)->type != REG_STAR_GLOB_STRING
+				&& vstack_bx(stack)->type != REG_STAR_GLOB_STRING) {
+			ERR("Unexpected register type for globbing pattern comparator\n");
+			ret = -EINVAL;
+			goto end;
+		}
+		break;
+	}
+
 	case FILTER_OP_EQ_S64:
 	case FILTER_OP_NE_S64:
 	case FILTER_OP_GT_S64:
@@ -609,6 +660,7 @@ int validate_instruction_context(struct bytecode_runtime *bytecode,
 			goto end;
 
 		case REG_STRING:
+		case REG_STAR_GLOB_STRING:
 			ERR("Unary op can only be applied to numeric or floating point registers\n");
 			ret = -EINVAL;
 			goto end;
@@ -722,6 +774,7 @@ int validate_instruction_context(struct bytecode_runtime *bytecode,
 
 	/* load from immediate operand */
 	case FILTER_OP_LOAD_STRING:
+	case FILTER_OP_LOAD_STAR_GLOB_STRING:
 	{
 		break;
 	}
@@ -753,6 +806,7 @@ int validate_instruction_context(struct bytecode_runtime *bytecode,
 			goto end;
 
 		case REG_STRING:
+		case REG_STAR_GLOB_STRING:
 			ERR("Cast op can only be applied to numeric or floating point registers\n");
 			ret = -EINVAL;
 			goto end;
@@ -852,7 +906,7 @@ int validate_instruction_all_contexts(struct bytecode_runtime *bytecode,
 	node = cds_lfht_iter_get_node(&iter);
 	if (node) {
 		mp_node = caa_container_of(node, struct lfht_mp_node, node);
-		
+
 		dbg_printf("Filter: validate merge point at offset %lu\n",
 				target_pc);
 		if (merge_points_compare(stack, &mp_node->stack)) {
@@ -936,6 +990,8 @@ int exec_insn(struct bytecode_runtime *bytecode,
 	case FILTER_OP_LT_STRING:
 	case FILTER_OP_GE_STRING:
 	case FILTER_OP_LE_STRING:
+	case FILTER_OP_EQ_STAR_GLOB_STRING:
+	case FILTER_OP_NE_STAR_GLOB_STRING:
 	case FILTER_OP_EQ_S64:
 	case FILTER_OP_NE_S64:
 	case FILTER_OP_GT_S64:
@@ -1113,6 +1169,19 @@ int exec_insn(struct bytecode_runtime *bytecode,
 		break;
 	}
 
+	case FILTER_OP_LOAD_STAR_GLOB_STRING:
+	{
+		struct load_op *insn = (struct load_op *) pc;
+
+		if (vstack_push(stack)) {
+			ret = -EINVAL;
+			goto end;
+		}
+		vstack_ax(stack)->type = REG_STAR_GLOB_STRING;
+		next_pc += sizeof(struct load_op) + strlen(insn->data) + 1;
+		break;
+	}
+
 	case FILTER_OP_LOAD_S64:
 	{
 		if (vstack_push(stack)) {
