@@ -35,6 +35,7 @@
 #include <time.h>
 #include <assert.h>
 #include <signal.h>
+#include <limits.h>
 #include <urcu/uatomic.h>
 #include <urcu/futex.h>
 #include <urcu/compiler.h>
@@ -52,7 +53,7 @@
 #include "tracepoint-internal.h"
 #include "lttng-tracer-core.h"
 #include "compat.h"
-#include "../libringbuffer/tlsfixup.h"
+#include "../libringbuffer/rb-init.h"
 #include "lttng-ust-statedump.h"
 #include "clock.h"
 #include "../libringbuffer/getcpu.h"
@@ -533,6 +534,19 @@ int get_constructor_timeout(struct timespec *constructor_timeout)
 	return 1;
 }
 
+static
+void get_allow_blocking(void)
+{
+	const char *str_allow_blocking =
+		lttng_getenv("LTTNG_UST_ALLOW_BLOCKING");
+
+	if (str_allow_blocking) {
+		DBG("%s environment variable is set",
+			"LTTNG_UST_ALLOW_BLOCKING");
+		lttng_ust_ringbuffer_set_allow_blocking();
+	}
+}
+
 static
 int register_to_sessiond(int socket, enum ustctl_socket_type type)
 {
@@ -1673,6 +1687,8 @@ void __attribute__((constructor)) lttng_ust_init(void)
 
 	timeout_mode = get_constructor_timeout(&constructor_timeout);
 
+	get_allow_blocking();
+
 	ret = sem_init(&constructor_wait, 0, 0);
 	if (ret) {
 		PERROR("sem_init");
